#![no_std]
#![no_main]
#![feature(allocator_api)]
#![allow(unused_imports)]

use gila::arch::current::display;
use gila::arch::current::*;
use gila::boot::*;
use gila::log::*;
use gila::memory;
use gila::memory::alloc::*;
use gila::panic;
use gila::params;
use gila::params::get_kernel_params;
use gila::process;

#[unsafe(no_mangle)]
unsafe extern "C" fn main() -> ! {
    // Assert supported bootloader version
    assert!(BASE_REVISION.is_supported());

    // Local logger fn
    fn log(level: LogLevel, msg: &str) {
        LOGGER.lock().log(level, msg);
    }

    // Set up some stuff based on kernel params if we get any
    if let Some(kernel_file_response) = FILE_REQUEST.get_response() {
        let kernel_parameters = get_kernel_params(kernel_file_response.file().cmdline());

        // Set up logging level from params
        if let Some(level) = kernel_parameters.get("-loglevel") {
            if let Ok(parsed_level) = LogLevel::try_from(level.as_str()) {
                LOGGER.lock().level = parsed_level;
            }
        }
        // Add subscribers to logger
        if let Some(device) = kernel_parameters.get("-logdev") {
            let log_device_list: vec::Vec<&str> = device.split(',').collect();
            if log_device_list.contains(&"display") {
                // Append display console to log subs
            }
            if log_device_list.contains(&"serial") {
                // Append serial console to log subs
            }
            log(LogLevel::INFO, "Configured kernel logging devices.")
        }

        {
            let kernel_cmdline_string =
                string::String::from_utf8_lossy(kernel_file_response.file().cmdline());
            log(
                LogLevel::INFO,
                &format!("Kernel cmdline: {}", kernel_cmdline_string),
            );
            drop(kernel_cmdline_string);
        }

        {
            let kernel_path = string::String::from_utf8_lossy(kernel_file_response.file().path());
            log(
                LogLevel::INFO,
                &format!("Kernel file path: {}", kernel_path),
            );
            drop(kernel_path);
        }
    }

    log(LogLevel::INFO, "Log devices configured. Booting `gila`.");
    log(LogLevel::INFO, "Trans Rights!");

    let _smp_response = SMP_REQUEST.get_response();

    match _smp_response {
        None => log(
            LogLevel::ERROR,
            "SMP response not received. Multiprocessing is disabled.",
        ),
        Some(resp) => {
            log(
                LogLevel::INFO,
                "SMP response received. Multiprocessing enabled.",
            );
            log(
                LogLevel::INFO,
                &format!("{} CPUs found.", resp.cpus().len()),
            );
        }
    }

    loop {
        unsafe {
            asm::halt();
        }
    }
}
